# datahub-restful-core [![NPM version](https://img.shields.io/npm/v/datahub-restful-core.svg?style=flat)](https://www.npmjs.com/package/datahub-restful-core) [![NPM monthly downloads](https://img.shields.io/dm/npm/datahub-restful-core.svg?style=flat)](https://npmjs.org/package/datahub-restful-core) [![NPM total downloads](https://img.shields.io/npm/dt/datahub-restful-core.svg?style=flat)](https://npmjs.org/package/datahub-restful-core) [![Linux Build Status](https://img.shields.io/travis/jonschlinkert/datahub-restful-core.svg?style=flat&label=Travis)](https://travis-ci.org/jonschlinkert/datahub-restful-core) [![Windows Build Status](https://img.shields.io/appveyor/ci/jonschlinkert/datahub-restful-core.svg?style=flat&label=AppVeyor)](https://ci.appveyor.com/project/jonschlinkert/datahub-restful-core)

##koa v1 上导出 mysql restful 接口的组件

[English ver](./README.md)

# Usage
```javascript

const koa = require('koa');

const {DatahubRestfulCore} = require('../index');
let port = process.env.PORT || 3003;
let host = process.env.IP || '127.0.0.1';

const core = new DatahubRestfulCore({
    port, host,
    name: 'demo',
    endpoint: "/",
    options:
        {
            client: 'mysql',
            connection: {
                host: '127.0.0.1',
                user: 'root',
                password: '',
                database: 'datahub',
                supportBigNumbers: true,
                bigNumberStrings: true,
                multipleStatements: true
            },
            acquireConnectionTimeout: 60000,
            pool: {min: 0, max: 2}
        }
});

app = core.mount(koa());

app.listen(port, host, function () {
    console.log('Datahub started http://%s:%s', host, port);
});

//curl -v http://127.0.0.1:3003/demo/

```

# 已支持以下功能:-)

- 路由规则
- Restful 接口 
- POST/GET/PUT/DELETE
- 分页
- 排序
- 复合主键
- 自定义SQL
- 搜索
- or 条件
- 前后处理器
- not查询
- in,not in查询
- between,not between查询
- null值，空值查询
- 指定字符集
  
# 其他功能

- 直接执行router逻辑
- 导出为xls
- 导入

# URL设计

## API部分
 - 所有数据库链接: [your ip:port/api/v1/databases](your ip:port/api/v1/databases)
 - 特定数据库链接的所有schema: [your ip:port/api/v1/{db}](your ip:port/api/v1/{db})
 - 指定字符集: _charset参数
 - 特定schema下的所有表 : [your ip:port/api/v1/{db}/{schema}](your ip:port/api/v1/{db}/{schema})
 - 特定schema下的某个表 : [your ip:port/api/v1/{db}/{schema}/{table}](your ip:port/api/v1/{db}/{schema}/{table})
 该url endpoint提供此table的CRUD方法,以RESTFUL形式导出
 - query: GET to [your ip:port/api/v1/{db}/{schema}/{table}?{k}={v}](your ip:port/api/v1/{db}/{schema}/{table}?{k}={v})
    - 支持附加操作符以实现大于,小于,模糊搜索等,
    以冒号指定操作符
    如contains: [your ip:port/api/v1/{db}/{schema}/{table}?{k}:contains={v}](your ip:port/api/v1/{db}/{schema}/{table}?{k}:contains={v}),则会列出k列中包含v的所有记录
    - contains : like %keyword% 包含keyword
    - startswith: like keyword% 以keyword开头
    - endswith : like %keyword 以keyword结尾
    - gt 或 >:  大于
    - lt 或 < :  小于- gte :  大于等于
    - lte : 小于等于
    - 分页: page=0 （0代表第一页）pageSize=3（单页记录数）
    - 排序: _sort=-policy_name,id 按policy_name列按降序排和按id升序排,
    - Or: [your ip:port/api/v1/{db}/{schema}/{table}?k=v&{or:k2}={v2}](your ip:port/api/v1/{db}/{schema}/{table}?k=v&{or:k2}={v2})
    - not查询: 
    - isnull:  为NULL值
    - isnotnull: 不为NULL值
    - isempty: 为空字符串
    - isnotempty: 不为为空字符串
    - in / notin: 参数为数组形式 `somecolumn:in=a,b,c` 或者`somecolumn:in=a&somecolumn:in=b&somecolumn:in=c`
    - between / notbetween: 参数为数组形式 `somecolumn:between=1,100`
    - 嵌套或者其他复杂的sql查询，请使用复杂查询接口
    - 返回结果示例
    ```json
    {"content":[{"id":83,"wifi_id":85,"policy_name":"wifi83_-78486031","status":-1,
    "create_time":"2016-05-12T12:43:19.000Z"},{"id":70,"wifi_id":82,"policy_name":"wifi",
    "status":-1,"create_time":"2016-05-11T02:57:22.000Z"},
    {"id":75,"wifi_id":82,"policy_name":"wifi",
    "status":1,"create_time":"2016-05-11T03:56:19.000Z"}],
    "page":{"size":30,"totalElements":3,"totalPages":1,"number":0}}
    ```
 - 复杂查询：
 - get: GET [your ip:port/api/v1/{db}/{schema}/{table}/{id}](your ip:port/api/v1/{db}/{schema}/{table}/{id})
 - get(复合主键): GET [your ip:port/api/v1/{db}/{schema}/{table}/{key1}={a};{key2}={b}](your ip:port/api/v1/{db}/{schema}/{table}/{key1}={a};{key2}={b})
 - create: JSON POST to [your ip:port/api/v1/{db}/{schema}/{table}](your ip:port/api/v1/{db}/{schema}/{table})
 - update: JSON PUT to [your ip:port/api/v1/{db}/{schema}/{table}/{id}](your ip:port/api/v1/{db}/{schema}/{table}/{id})
 - delete: DELETE [your ip:port/api/v1/{db}/{schema}/{table}/{id}](your ip:port/api/v1/{db}/{schema}/{table}/{id})
 - patch(Partial update):  PATCH [your ip:port/api/v1/{db}/{schema}/{table}/{id}](your ip:port/api/v1/{db}/{schema}/{table}/{id})
 - 自定义SQL: GET [your ip:port/api/v1/{db}/{schema}/_q/{key}](your ip:port/api/v1/{db}/{schema}/_q/{key})
 - 自定义SQL分页支持: GET [your ip:port/api/v1/{db}/{schema}/_q/{key}/pagination](your ip:port/api/v1/{db}/{schema}/_q/{key}/pagination)
  - 前后处理器
    - 前后处理器是执行api之前或之后的逻辑代码,配置页面在
    - [your ip:port/admin/hooks](your ip:port/admin/hooks)
    - 前处理器 * [your ip:port/api/v1/*?_hook:before=someHook1](your ip:port/api/v1/*?_hook:before=someHook1)
    - 后处理器 * [your ip:port/api/v1/*?_hook:after=someHook1](your ip:port/api/v1/*?_hook:after=someHook1)
           
 - 直接执行SQL: GET/POST sql到 [your ip:port/api/v1/{db}/sqlExecutor](your ip:port/api/v1/{db}/sqlExecutor)
 - 直接执行SQL分页: GET/POST sql到 [your ip:port/api/v1/{db}/sqlExecutor/pagination](your ip:port/api/v1/{db}/sqlExecutor/pagination)
  - 直接执行SQL UI : [your ip:port/api/v1/{db}/sqlExecutor/form](your ip:port/api/v1/{db}/sqlExecutor/form)
  - 都支持以 ?sql=`select * from t_x_table where f_some_field = :value`&value=查询值 的queryString形式传参调用
         

 - 直接执行router逻辑
     [your ip:port/api/v1/vm/{someHook1}](your ip:port/api/v1/vm/{someHook1})
               someHook1 : 钩子标志
               钩子里提供的环境有require, console, knexHelper
               可require的库有request, request-promise
   
# KnexHelper对象
    - getInternalApiUrl() 获取内部api地址`http://${host}:${port}${endpoint}`, 可以内网url形式调用data的restful api,去除无谓的dns和lb损耗
    - * findConnectionByDbkey(dbkey) 获取数据库cdb_x形式的cdbId
    - getDatabaseConnection(cdbId) 以获取数据库链接conn对象

# Test

```javascript
tnpm i -g mocha
tnpm run test
```
