'use strict';

class BasicAuthorizationProvider {

    constructor(options) {

        this.options = options || {};

        this.authorizator = this.options.authorizator || {

                authorize: function alwayPass(authInfo) {
                    authInfo.authorizated = true;
                    return authInfo;
                }

            }
    }

    adaptToKoa(context) {

        return {
            authorize: function () {
                return this._authorize(context.headers['authorization'])
            }.bind(this)
        };
    }

    extractAuthInfo(authorizeHeader) {

        let username = '';
        let password = '';

        if (authorizeHeader && authorizeHeader.search('Basic') == 0) {
            let encoded = authorizeHeader.substring(6, authorizeHeader.length - 1);
            username = Buffer.from(encoded, 'base64').toString().replace(/([^:]*):?([^:]*)/, '$1');
            password = Buffer.from(encoded, 'base64').toString().replace(/([^:]*):?([^:]*)/, '$2');
        }

        return {username: username, password: password};
    }

    _authorize(authorizeHeader) {
        let authInfo = this.extractAuthInfo(authorizeHeader);
        authInfo = this.authorizator.authorize(authInfo);
        return authInfo;
    }
}

module.exports = BasicAuthorizationProvider;