'use strict';
const xlsx = require('node-xlsx');
const decodeHtml = require('./html-decoder');


class ExportSupport {

    constructor() {
        this.config = {};
    }

    outputConfig(config) {
        //exporter
        return new Object();
    }


    /**
     *
     * @param rows data records
     */
    export(rows, sheetName) {

        let headerWrote = false;
        let headers = [];

        rows = rows.map((e) => {
            if (typeof e === 'object') {
                let a = [];
                for (let k in e) {
                    a.push(e[k]);
                }
                if(!headerWrote){
                    for (let k in e) {
                        headers.push(k);
                    }
                    headerWrote = true;
                }

                return a;
            } else {
                return e;
            }
        });

        rows.unshift(headers);

        return xlsx.build([{name: sheetName || "sheet1", data: rows}]);
    }

    static middleware() {
        let exportSupport = new ExportSupport();
        return function *(next) {
            let url = this.originalUrl.replace(/(\?.*)/g, '');
            if (url.search(/\.xls$|\.xlsx$/i) > -1) {
                this.query['pageSize'] = this.query['pageSize'] || 999;
                let fileExt = url.split('.').pop();
                let filename = this.query._exportFilename || 'export';
                delete this.query['_exportFilename'];

                if ('sql' in this.query) {
                    this.query['sql'] = decodeHtml(this.query['sql']);
                }

                yield next;
                if(this.status == 400) {
                    return;
                }
                
                this.set('Content-Disposition', 'attachment; filename=' + encodeURI(filename) + '.' + fileExt);
                let res = exportSupport.export(this.body.content ? this.body.content : this.body);
                this.body = res;

            }else if (url.search(/\.sql$/i) > -1) {
                this.query['pageSize'] = this.query['pageSize'] || 999;
                let fileExt = url.split('.').pop();
                let filename = this.query._exportFilename || 'data';
                delete this.query['_exportFilename'];

                yield next;
                if(this.status == 400) {
                    return;
                }

                this.set('Content-Disposition', 'attachment; filename=' + encodeURI(filename) + '.' + fileExt);
                let res =this.body.content ? this.body.content : this.body;
                let sql = '';
                if(res.length){
                    let fields = Object.keys(res[0]).map(key=>{return `\`${key}\``}).join(',')
                    let values = res.map(e=>{
                        return '('+Object.values(e).map(value=>{return `\'${(value+'').replace('0000-00-00','2000-01-01').replace(/([\w\-])(T)([\w:\.]*)\.000Z/,'$1 $3')}\'`})+')';
                    }).join(',\n');
                    sql = `insert into ${this.params.state}.${this.state.entity} (${fields}) values ${values};`;
                }
                this.body = sql;

            } else {
                yield next;
            }
        }
    }
}

module.exports = ExportSupport;