/*
 * @Descripttion: 
 * @version: 
 * @Author: sueRimn
 * @Date: 2019-08-15 11:13:44
 * @LastEditors: sueRimn
 * @LastEditTime: 2019-08-16 15:52:21
 */
'use strict';
const Parser = require('./pegjs/parser')
const debug = require('debug')('datahub-core:sqlParser:debug');
const toSql = require('./sqlMysqlDialet');

//sql to AST
class SqlParser {

  constructor() {
    this.parser = new Parser();
  }

  parse (sql) {
    return this.parser.parse(sql);
  }

  toSql (ast) {
    return toSql(ast);
  }

  removeEmptyParams (ast, params, db) {

    if (!params) {
      params = new Object();
    }

    if (ast.type === 'unary_expr' && ast.expr.type === 'select') {

      debug('type is select.');
      if (ast.expr.from) {
        debug('has from.');
        //递归找表达式
        for (let ele of ast.expr.from) {
          if (!ele.db && ele.table && db) {
            ele.db = db;
          }

          if (ele.expr) {
            ele.expr = this.removeEmptyParams(ele.expr, params);
          }
        }
      }

      if (ast.expr.where) {
        ast.expr.where = this.removeEmptyParams(ast.expr.where, params);
      }
    }

    if (ast.type === 'select') {

      debug('type is select.');
      if (ast.from) {
        debug('has from.');
        //递归找表达式
        for (let ele of ast.from) {
          if (!ele.db && ele.table && db) {
            ele.db = db;
          }
          if (ele.expr) {
            ele.expr = this.removeEmptyParams(ele.expr, params);
          }
        }
      }

      if (ast.where) {
        ast.where = this.removeEmptyParams(ast.where, params);
      }
    }

    if (ast.type === 'binary_expr') {
      debug('is binary_expr', ast.right);

      if (ast.right && ast.right.type === 'param') {
        debug(ast.value);
        if (!this.hasKey(params, ast.right.value)) {
          debug('removing', ast);
          ast.operator = '=';
          ast.left = { type: 'number', value: 1 };
          ast.right = { type: 'number', value: 1 };
        } else {
          let value = params[ast.right.value];
          if (ast.operator == 'LIKE') {
            value = this.autoCompleteWildCard(value);
          }
          ast.right = { type: typeof value, value: value };
        }
      } else if (ast.right && ast.right.type === 'expr_list') {
        for (let i = 0, l = ast.right.value.length; i < l; i++) {
          if (ast.right.value[i].type === 'param' && ast.operator === 'IN') {
            if (ast.right.value[i].value in params) {
              ast.right.value[i].value = `:${ast.right.value[i].value}`;
            } else {
              ast.operator = '=';
              ast.left = { type: 'number', value: 1 };
              ast.right = { type: 'number', value: 1 };
            }
          } else {
            ast.right.value[i] = this.removeEmptyParams(ast.right.value[i], params);
          }
        }
      } else {
        ast.left = this.removeEmptyParams(ast.left, params);
        ast.right = this.removeEmptyParams(ast.right, params);
      }
    }
    if (ast._next) {
      ast._next = this.removeEmptyParams(ast._next);
    }
    if (ast.right && ast.right.type === 'function' && ast.right.args && ast.right.args.value) {
      const right = ast.right
      if (Array.isArray(right.args.value) && right.args.value.length) {
        const arr = right.args.value;
        const exist = arr.every(i => {
          if (i.type === 'param' && params[i.value] === undefined) {
            return false;
          }
          return true;
        });
        if (!exist) {
          ast.operator = '=';
          ast.left = { type: 'number', value: 1 };
          ast.right = { type: 'number', value: 1 };
        }
        arr.forEach((i, index) => {
          arr[index] = this.removeEmptyParams(i, params);
        });
        right.args.value = arr;
      }
    }
    if (ast.type === 'param') {
      if (params[ast.value]) {
        if (typeof params[ast.value] === 'string') {
          ast.value = `'${params[ast.value]}'`;
        } else {
          ast.value = params[ast.value];
        }
      }
    }
    return ast;
  }

  autoCompleteWildCard (value) {
    value = value + '';
    let wildcardPosition = value.search('%');
    if (wildcardPosition != 0 && wildcardPosition != value.length - 1) {
      value = `%${value}%`;
    }
    return value;
  }

  hasKey (obj, key) {
    if (obj.prototype && obj.prototype.hasOwnProperty) {
      return obj.hasOwnProperty(key);
    } else {
      return obj[key];
    }
  }

  /**
   * 参数替换,干掉不存在的参数
   * @param sql
   * @param param
   * @returns {*}
   */
  cleanAndReplaceParams (sql, params, db) {
    sql = (sql.trim()).replace(/;+$/g, '');
    let isReadStmt = sql.search(/(select|desc|show|describe)/i) === 0;

    if (isReadStmt) {
      //only support read only at this moment
      let ast = this.removeEmptyParams(this.parse(sql), params, db);
      sql = this.toSql(ast);
    }
    return sql;
  }

  formatSql (sql) {
    sql = sql.replace(/(^\w):/g, '$1 :').replace(/;/g, '');
    return sql.replace(/\\\\ :/g, '\\\\:');
  }
}

module.exports = SqlParser;