'use strict';
const moment = require('moment');

const getDatetimeFields = function *(connection, schema, entity) {
    let columns = yield metaInfo(connection, schema, entity);

    let result = [];

    columns.filter((e) => e.DATA_TYPE === 'datetime'
    || e['DATA_TYPE'] === 'date'
    || e['DATA_TYPE'] == 'time'
    || e['DATA_TYPE'] == 'datetime'
    || e['DATA_TYPE'] == 'timestamp')
        .map((e) => e['COLUMN_NAME']).forEach((e) => result.push(e));

    return result;
}

const metaInfo = function *(connection, schema, entity) {
    return yield connection(entity)
        .select('*').from('information_schema.columns')
        .where({'table_name': entity, 'table_schema': schema});
}

/**
 * 设置默认时间
 * @param connection
 * @param schema
 * @param entity
 * @param body
 * @returns {*}
 */
const transformDatetimeFields = function *(connection, schema, entity, body) {
    let fields = yield getDatetimeFields(connection, schema, entity);

    for (let key in body) {
        if (fields.indexOf(key) > -1) {
            //空则为设置为当前时间
            body[key] = parseToDate(body[key]);
        }
    }

    return body;
}

const parseToDate = function (str) {
    return str ? moment(str).toDate() : new Date();
}

module.exports = {
    getDatetimeFields: getDatetimeFields,
    metaInfo: metaInfo,
    transformDatetimeFields: transformDatetimeFields,
    parseToDate: parseToDate
}