'use strict';

const debug = require('debug')('krh:route');
const Route = require('koa-router');

const mm = require('micromatch');

/**
 * Export databases
 * @type {Router|exports|module.exports}
 */
const databaseRouter = ({endpoint}) => {

    let router = new Route({prefix: endpoint});

    router.param('database', function* (database, next) {
        if(this.state.database) {
            return yield next;
        }

        debug('database param', database);
        //resolve path to this.state.database for further using
        if (!database) {

            this.body = '缺少mysql实例标识';
            return this.status = 400;

        } else {

            let possibleConnectionId = yield this.knexHelper.findConnectionByDbkey(database);

            if (possibleConnectionId) {
                database = possibleConnectionId;
            }

            if(!this.state.privilege.table[database] && !this.state.privilege.freeDatabase.has(database)) {
                this.body = '无权限';
                return this.status = 400;
            }

            let databases = this.knexHelper.getMetaDataCache();
            if (!databases.has(database) && database != 'databases') {
                this.status = 404;
                if (this.render){
                    yield this.render('databaseNotFound', {database: database});
                }

            } else {

                this.state.dataConfig = this.knexHelper.getDatabaseConfig(database);
                this.state.database = database;
                this.state.charset = this.query._charset || 'utf8';

                this.state.connection = this.state.charset !== 'utf8'
                    ? this.knexHelper.getDatabaseConnection(`${this.state.database}_${this.state.charset}`)
                    : this.knexHelper.getDatabaseConnection(database)
                ;

                if (this.state.connection === void 0) {
                    this.state.connection = this.knexHelper.addDatabaseConnection(this.state.dataConfig, `${this.state.database}_${this.state.charset}`, this.state.charset);
                }

                if ('_lego_schema' in this.query) {
                    this.state.schema = this.query._lego_schema;
                    yield this.state.connection.raw('use :schema:', {schema: this.query._lego_schema});
                }

                yield next;
                if (this.state.connection.client.pool.used.length) {
                    debug('connection used?', this.state.connection.client.pool);
                }

            }
        }

    }).all('*', function* (next) {
        this.state.api = {
            endpoint
        };
        yield next;
    }).get('/databases/setting', function* () {
        yield next;
    }).get('/databases/rawTree', function* () {
        this.body = '该接口已不再支持';
        return;

        let result = yield this.knexHelper.getDatabasesTree();
        this.body = result;
    }).get('/databases/tree', function* () {
        this.body = '该接口已不再支持';
        return;

        let result = yield this.knexHelper.getDatabasesTree();
        let accessWhiteList = yield this.knexHelper.getMetaConnection().select('*').from('api_access_whitelist');

        result.forEach((connection) => {

            let cdbKey = connection.cdb_key;

            connection.schemas.forEach((schema) => {

                let schemaName = schema.schema;

                schema.tables.forEach((table) => {
                    let targetUrl = '/' + [cdbKey, schemaName, table.table].join('/');
                    let grantAccess = accessWhiteList.filter((e) => mm.isMatch(targetUrl, e.api_url)).length > 0;
                    table.grantAccess = grantAccess;
                });

                schema.tables = schema.tables.filter((table) => {
                    return table.grantAccess;
                });
                schema.tables.forEach((table) => delete table['grantAccess']);

            });

            connection.schemas = connection.schemas.filter((schema) => {
                return schema.tables.length > 0;
            });

        });

        result = result.filter((connection) => connection.schemas.length > 0);

        this.body = result;

    }).get('/databases', function* () {
        const dbList = [];
        (yield this.knexHelper.getDatabaseConnectionConfigs()).forEach(element => {
            if(this.state.privilege.table[element.cdb_key] || !this.state.privilege.freeDatabase.has(element.cdb_key)) {
                element.user = '***';
                element.password = '***';
                dbList.push(element);
            }
        });
        this.body = dbList;
    }).post('/databases', function* () {
        this.body = '该接口已不再支持';
        return;

        //saving modules configuration
        let dbConfig = yield this.knexHelper.saveDatabaseConfig(this.request.body);
        yield this.knexHelper.polling();
        this.body = dbConfig;

    }).all('/:modules/(.*)', function* (next) {

        //Cascade modules to next chain.
        debug('Accessing modules', this.state.database);
        yield next;

    });

    return router;
}
module.exports = databaseRouter;