'use strict';

const debug = require('debug')('krh:route');
const route = require('koa-router');

/**
 * Export schemas according to database
 * @type {Router|exports|module.expor   ts}
 */
var router = new route();
router
    .param('schema', function *(schema, next) {
        if(this.state.schema) {
            return yield next;
        }

        debug('schema param', schema);
        //resolve path to state.schema for table usage
        if (!schema) {
            this.body = '缺少数据库schema';
            return this.status = 400;
        } else {
            if(!this.state.privilege.freeDatabase.has(this.state.database) && !this.state.privilege.table[this.state.database][schema]) {
                this.body = '无权限';
                return this.status = 400;
            }
            this.state.schema = this.state.schema || schema;
            yield this.state.connection.raw('use :schema:', {schema: this.state.schema});
            yield next;
        }
    }).get('/', function *() {

    //mysql cmd: show tables.
    let schemas = yield this.state.connection.raw('show databases');
    let host = [this.protocol,
        this.header['x-datahub-host'] || this.request.host].join('://');

    //joining schemas path and export readable href
    // filter mysql sys schemas
    this.body = schemas[0]
        .map(e => {
            debug(e, this.state.database);
            let path = [this.request.path, e.Database].join('/').replace(/\/\//g, '/');
            return {
                schema: e.Database,
                href: host + path
            }
        })
        .filter(e => ['information_schema', 'mysql', 'performance_schema', 'sys'].indexOf(e.schema) == -1)
        .filter(e => {
            if(!this.state.privilege.freeDatabase.has(this.state.database) && !this.state.privilege.table[this.state.database][e.schema]) {
                return false;
            }
            return true;
        });
})
    .all('/:schema/(.*)', function *(next) {
        /* 编码设置提升到database做单独pool处理 */
        //this.state.charset = this.query._charset || 'utf8';
        //delete this.query._charset;
        yield this.state.connection.raw('set names :charset', {charset: this.state.charset});
        yield this.state.connection.raw('use :schema:', {schema: this.state.schema});
        debug('Accessing schema', this.state.schema);
        yield next;
    });

module.exports = router;