'use strict';

const route = require('koa-router');
const importSupport = require('../core/http/import-support');
const HookExecutor = require('../middlewares/hook-executor');
const vm = require('vm');

/**
 * Vm executor
 * @type {Router|exports|module.exports}
 */
function routerFactory({knexHelper,endpoint}) {

    let hookExecutor = new HookExecutor({knexHelperInstance: knexHelper});

    let router = new route({prefix: endpoint});
    router.all('/vm/:hook'
        , importSupport()
        , function *(next) {

            let hookObj = hookExecutor.resolveHook({"_hook:around": this.params.hook});

            //share sandbox between hooks
            let sandBox = vm.createContext({
                "require": require,
                "console": console,
                "knexHelper": this.knexHelper,
                "setTimeout": setTimeout,
                module: {exports: {}}
            });

            try {
                yield hookExecutor.executeScript(hookObj.around, sandBox, this, next);
            } catch (err) {
                this.body = err;
            }

        });

    return router;

}

module.exports = routerFactory;