const request = require('request-promise');

const should = require('should');

const {app, url} = require('../examples/app')
process.env.debug = '*';
describe('crud', function () {

    let a, b;

    before((done) => {
        console.log('starting server\n');
        a = app.start(done)
    })


    describe('basic crud', function () {
        it("query", (done) => {
            request({url: `${url}/datahub/sql_access_log`, json: true}).then(resp => {
                resp.content.should.notNull;
                done();
            })
        })

        it("post", (done) => {
            let now = Date.now();
            request({
                method: 'POST', url: `${url}/datahub/sql_access_log`, json: true, body:
                    {
                        sql_mapping_id: 999,
                        ip: '127.0.0.1',
                        sql: 'hello',
                        access_time: now,
                        cost_time: 1,
                        query: '',
                        source: 'o'
                    }
            }).then(resp => {
                resp.sql_mapping_id.should.eql(999);
                b = resp;
                done();
            }).catch(err => {
                console.error(err);
                done(err);
            })
        })

        it("put", (done) => {
            let now = Date.now();
            request({
                method: 'PUT', url: `${url}/datahub/sql_access_log/${b.id}`, json: true, body:
                    {
                        sql_mapping_id: 998,
                        ip: '127.0.0.1',
                        sql: 'hello',
                        access_time: now,
                        cost_time: 1,
                        query: '',
                        source: 'o'
                    }
            }).then(resp => {
                resp.sql_mapping_id.should.eql(998);
                done();
            }).catch(err => {
                console.error(err);
                done(err);
            })
        })

        it("delete", (done) => {
            let now = Date.now();
            request({
                method: 'DELETE', url: `${url}/datahub/sql_access_log/${b.id}`, json: true
            }).then(resp => {
                resp[0].affects.should.eql(1);
                done();
            }).catch(err => {
                console.error(err);
                done(err);
            })
        })
    })

    describe('realword query', function () {
        let {content} = {
            "content": [{
                "id": 10,
                "sql_key": "aliasView",
                "sql": "select db_connection.host,\ndb_connection.port,\ndb_connection.user,\ndb_connection.password,\ndb_connection.created_at,\ndb_connection.updated_at,\ndb_key.* from datahub.db_connection left join datahub.db_key on db_connection.id = db_key.connection_id;",
                "connection_id": 3,
                "schema_name": "datahub"
            }, {
                "id": 11,
                "sql_key": "gethighrisklung_1",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where r.f_risk_num >= 0.5 and r.f_ai_code = 200 \nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 1",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 13,
                "sql_key": "gethighrisklung_45",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where r.f_risk_num >= 0.5 and r.f_ai_code = 200 \nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 45",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 14,
                "sql_key": "gethighrisklung_5",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\r\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\r\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where r.f_risk_num >= 0.5 and r.f_ai_code = 200 \r\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 5",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 15,
                "sql_key": "gethighrisklung_22",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\r\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\r\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where r.f_risk_num >= 0.5 and r.f_ai_code = 200 \r\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 22",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 16,
                "sql_key": "gethighrisklung_58",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\r\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\r\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where r.f_risk_num >= 0.5 and r.f_ai_code = 200 \r\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id =58",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 17,
                "sql_key": "gethighrisklung_7",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where (s.f_image_desc!='' or s.f_conclusion!='' )\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 7",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 18,
                "sql_key": "gethighrisklung_47",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where (s.f_image_desc!='' or s.f_conclusion!='' )\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 47",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 19,
                "sql_key": "gethighrisklung_49",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where (s.f_image_desc!='' or s.f_conclusion!='' )\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 49",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 20,
                "sql_key": "gethighrisklung_110",
                "sql": "select r.f_study_id, r.f_ai_mark_num,r.f_risk_num,s.f_hospital_id,s.f_study_type,s.f_patient_id,s.f_patient_name,\ns.f_patient_gender,s.f_study_name,s.f_study_date,s.f_image_desc,s.f_conclusion,s.f_redundance,s.f_patient_birthday,\ns.f_pathology_report,s.createdAt,s.updatedAt from t_study_dicom_airesult r, t_study s where (s.f_image_desc!='' or s.f_conclusion!='' )\nand r.f_ai_engine = 3 and  r.f_study_id = s.f_id and s.f_hospital_id = 110",
                "connection_id": 19,
                "schema_name": "db_ai_image"
            }, {
                "id": 21,
                "sql_key": "nanshan_2.3",
                "sql": "select * from t_study where f_study_type = 1 and f_hospital_id = 1 and f_study_date < '2017-11-30' order by f_id;",
                "connection_id": 35,
                "schema_name": "db_ai_image"
            }, {
                "id": 22,
                "sql_key": "wenzhou_2_3",
                "sql": "select * from t_study where f_study_type = 1 and f_hospital_id = 21 and f_study_date < '2017-12-28' and f_study_date >= '2017-12-20 00:00:00' order by f_id;",
                "connection_id": 35,
                "schema_name": "db_ai_image"
            }, {
                "id": 24,
                "sql_key": "all_news",
                "sql": "select id, title, icon, summary, source, pub_time from t_news order by position asc, pub_time desc;",
                "connection_id": 18,
                "schema_name": "db_internetplus_official_dev"
            }, {
                "id": 26,
                "sql_key": "all_formal_news",
                "sql": "select id, title, icon, summary, source, pub_time from t_news order by position asc, pub_time desc;",
                "connection_id": 20,
                "schema_name": "db_internetplus_official_dev"
            }, {
                "id": 27,
                "sql_key": "empty",
                "sql": "select * from t_user limit 0;",
                "connection_id": 39,
                "schema_name": "medical_record_data"
            }, {
                "id": 29,
                "sql_key": "getdata_1",
                "sql": "select f_province_name , f_city_code ,f_busi_type,f_channel,f_city_url,GROUP_CONCAT(f_order) as f_order , GROUP_CONCAT(f_service_name) as f_service_name , GROUP_CONCAT(f_logo_url) as f_logo_url , GROUP_CONCAT(f_busi_id ) as f_busi_id , GROUP_CONCAT(f_busi_url)  as f_busi_url ,f_operator ,f_update_time from db_healthcard_op_test.t_social_service_config GROUP BY f_city_code",
                "connection_id": 10,
                "schema_name": "db_healthcard_op_test"
            }, {
                "id": 30,
                "sql_key": "test",
                "sql": "SELECT t_proxy.f_id AS id, t_proxy.f_business_name AS businessName, t_proxy.f_func AS func, t_proxy.f_nick_name AS nickName, t_proxy.f_proxy_str AS proxyStr, t_proxy.f_request_object AS requestObject, t_proxy.f_response_object AS responseObject, t_proxy.f_description AS description, t_proxy.f_version AS version, t_proxy.f_type AS proxy_type, t_proxy.f_doc_url AS docURL FROM t_proxy JOIN t_business_proxy ON t_business_proxy.f_business_id in (32, 51) AND t_business_proxy.f_proxy_id = t_proxy.f_id",
                "connection_id": 10,
                "schema_name": "db_buffet_test"
            }, {
                "id": 31,
                "sql_key": "log_config",
                "sql": "select t_business.f_id, t_business.f_name, t_business_log_config .f_app_name from t_business left join t_business_log_config on t_business.f_id = t_business_log_config.f_business_id",
                "connection_id": 184,
                "schema_name": "db_buffet_test"
            }, {
                "id": 32,
                "sql_key": "search_job",
                "sql": "select * from db_daf_engine.t_job_record \nwhere f_job_module like :job_module \nor f_job_name like :job_name\norder by f_start_time desc",
                "connection_id": 61,
                "schema_name": "db_daf_engine"
            },{sql:'select+*+from+t_demo_article&_lego_schema=db_hlego_demo_test&page=0&pageSize=10&f_title:contains=很开心s'}],
            "page": {"size": "9999", "totalElements": "21", "totalPages": 1, "number": 0},
            "size": "9999",
            "totalElements": "21",
            "totalPages": 1,
            "number": 0
        }
        let sqls = content.map(e => e.sql);
        sqls.forEach(function (sql) {
            it(`test mapping ${sql}`, function (done) {
                request({
                    method: 'GET', url: `${url}/sqlExecutor`, qs: {sql}, json: true
                }).then(resp => {
                    resp.should.notNull;
                    done();
                }).catch(err => {
                    console.error(err);
                    done(err);
                })
            });
        })
    })

    after((done) => {
        console.log('closing server\n');
        a.close(done)
    });
})



