/*
 * @Descripttion:
 * @version:
 * @Author: sueRimn
 * @Date: 2019-08-15 11:13:44
 * @LastEditors: sueRimn
 * @LastEditTime: 2019-08-16 16:40:12
 */
const sqlParser = require('../core/sql/sqlParser');
const should = require('should');
const parser = new sqlParser();
const fs = require("fs");
//TODO test no params
describe(`SqlParser removeEmptyParams`, function () {
  it(`test hhhh`, function () {
    let str = `select AES_DECRYPT(:Fcardno,'abc123') as 'USER_NAME' from USER_INFO`
    str = `SELECT Fauto_id,Finspect_name,Fsend_time,Freport_diagnosis,Fupdate_time,Finsert_time FROM 
    t_breport WHERE Fauto_id=:Fauto_id 
    AND Fcardno = aes_encrypt (:Fcardno, '&^Askdf!@132nM' ) 
    AND Finspect_name LIKE :Finspect_name 
    AND Fsend_time LIKE :Fsend_time 
    AND Freport_diagnosis LIKE :Freport_diagnosis 
    AND Fupdate_time >= :Fupdate_time_gte 
    AND Fupdate_time <= :Fupdate_time_lte 
    AND Finsert_time >= :Finsert_time_gte 
    AND Finsert_time <= :Finsert_time_lte`
    let ast = parser.parse(str);
    // console.log(JSON.stringify(astn))
    ast = parser.removeEmptyParams(ast, { Fauto_id: 1, Fcardno: '3' });
    console.log(parser.toSql(ast));
  });

  it(`test 1`, function () {
    let ast = parser.parse("select r.*, news.`status` from rel_news_cate r left join news on news.id = r.news_id where r.is_hot = :i and news.`status` = :s and r.cate_id = :cate_id order by r.rank asc");
    ast = parser.removeEmptyParams(ast);
    console.log(parser.toSql(ast));
  });

  it(`should keep fixed conditions`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = 1');
    ast = parser.removeEmptyParams(ast);
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 1');
  });

  it(`should support string parameter`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a');
    ast = parser.removeEmptyParams(ast, { 'a': '123' });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = \'123\'');
  });

  it(`should support number parameter`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123');
  });

  it(`should remove unmatched conditions`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a');
    ast = parser.removeEmptyParams(ast);
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE 1 = 1');
  });

  it(`should keep matched conditions`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123');
  });

  it(`should keep matched conditions except unmatched ones`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789');
    ast = parser.removeEmptyParams(ast, {});
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE 1 = 1 AND `y` = 789');
  });

  it(`should keep both matched and fixed conditions`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789');

  });

  it(`should keep both matched and fixed conditions withour unmatched`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :b');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789 AND 1 = 1');

  });

  it(`should replace multi-params`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :b');
    ast = parser.removeEmptyParams(ast, { 'a': 123, 'b': 'xyz' });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789 AND `z` = \'xyz\'');

  });

  it(`should replace same param names`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :a');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789 AND `z` = 123');

  });

  it(`should replace when duplicated param keys`, function () {
    let ast = parser.parse('SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :a');
    ast = parser.removeEmptyParams(ast, { 'a': 123, 'a': 456 });
    parser.toSql(ast).should.eql('SELECT * FROM `a` WHERE `x` = 456 AND `y` = 789 AND `z` = 456');

  });


  it(`should support sub-query`, function () {
    let ast = parser.parse('SELECT * FROM (SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :a) AS S');
    ast = parser.removeEmptyParams(ast, { 'a': 123 });
    parser.toSql(ast).should.eql('SELECT * FROM (SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789 AND `z` = 123) AS `S`');

  });

  it(`should support complex sub-query`, function () {
    let ast = parser.parse('SELECT * FROM (SELECT * FROM `a` WHERE `x` = :a AND `y` = 789 AND `z` = :a) AS S WHERE o > :m');
    ast = parser.removeEmptyParams(ast, { 'a': 123, 'm': `789` });
    parser.toSql(ast).should.eql('SELECT * FROM (SELECT * FROM `a` WHERE `x` = 123 AND `y` = 789 AND `z` = 123) AS `S` WHERE `o` > \'789\'');

  });

  it(`should not support delete`, function () {
    let sql = parser.cleanAndReplaceParams('Delete from table_a where id=:id');
    sql.should.eql('Delete from table_a where id=:id');

  });

  it('should parse this demo: EXISTS', function () {
    let ast = parser.parse(`select SUM(a.id) from a where EXISTS(select * from b where b.categoryID=:categoryID)`);

    ast = parser.removeEmptyParams(ast, { "categoryID": "469" });
    let sql = parser.toSql(ast);
    console.log(sql);
  });

  it('should complete wildcards', function () {
    parser.autoCompleteWildCard('123').should.eql('%123%');
  });

  it('should not complete wildcards', function () {
    parser.autoCompleteWildCard('%123').should.eql('%123');
  });

  it('should not complete wildcards', function () {
    parser.autoCompleteWildCard('123%').should.eql('123%');
  });

  it('should not complete wildcards', function () {
    parser.autoCompleteWildCard('%123%').should.eql('%123%');
  });

  it('should replace all wheres', function () {
    let ast = parser.parse("select t1.* from t1 where t1.id = :t1id union all select t2.* from t2 where t2.id = :t2id");
    ast = parser.removeEmptyParams(ast);
    console.log(parser.toSql(ast));
  });

  it('should parse join', function () {
    let sql = `select db_connection.host, db_connection.port, db_connection.'user', db_connection.password, db_connection.created_at, db_connection.updated_at, db_key.* from datahub.db_connection left join datahub.db_key on db_connection.id = db_key.connection_id`
    let ast = parser.parse(sql);
    ast = parser.removeEmptyParams(ast);
    console.log(parser.toSql(ast));
  })

  it('should parse concat', function () {
    let sql = `select concat(id, name) as cid from some_table`
    let ast = parser.parse(sql);
    ast = parser.removeEmptyParams(ast);
    console.log(parser.toSql(ast));
  })

  it('should parse group concat', function () {
    let sql = `select group_concat(t1.check_status) 'status' from t_wx_account_check_result t1 group by t1.wx_account_id`
    let ast = parser.parse(sql);
    ast = parser.removeEmptyParams(ast);
    console.log(parser.toSql(ast));
  })

    it('should parse union all 3', function () {
        let sql = `select Fauto_id, Fdrug_hash_id, Fdrug_name, Fspecification, Fmanufacturer, Fdrug_org_id, 10000 as Fhospital_id from db_aid_drug_product.t_drug_product_10000 UNION ALL
         select Fauto_id, Fdrug_hash_id, Fdrug_name, Fspecification, Fmanufacturer, Fdrug_org_id, 10001 as Fhospital_id from db_aid_drug_product.t_drug_product_10001 UNION ALL
          select Fauto_id, Fdrug_hash_id, Fdrug_name, Fspecification, Fmanufacturer, Fdrug_org_id, 10003 as Fhospital_id from db_aid_drug_product.t_drug_product_10003`;
        let ast = parser.parse(sql);
        ast = parser.removeEmptyParams(ast);
        console.log(parser.toSql(ast));
    })

});
